{% if telegram_credentials_from_env %}
<p><span class="badge text-bg-warning">Управляется через окружение</span> — <code>TG_API_ID</code> и <code>TG_API_HASH</code> заданы во внешней конфигурации, поэтому эта форма скрыта.</p>
{% else %}
{% if is_configured %}
<p><span class="badge text-bg-success">Настроено</span> — API credentials сохранены.</p>
{% else %}
<p><span class="badge text-bg-warning">Не настроено</span> — для работы приложения необходимо указать Telegram API credentials.</p>
{% endif %}

<details {% if not is_configured %}open{% endif %}>
    <summary>Как получить API ID и API Hash</summary>
    <ol class="mt-2">
        <li>Перейдите на <a href="https://my.telegram.org/apps" target="_blank" rel="noopener">my.telegram.org/apps</a></li>
        <li>Авторизуйтесь с вашим номером телефона Telegram</li>
        <li>Заполните форму создания приложения (название и короткое имя — любые)</li>
        <li>Скопируйте <strong>App api_id</strong> и <strong>App api_hash</strong></li>
    </ol>
</details>

<form method="post" action="/settings/save-credentials" class="mt-3" data-busy>
    <div class="row g-3 mb-3">
        <div class="col-12 col-md-6">
            <label for="api_id" class="form-label">API ID</label>
            <input type="text" class="form-control" id="api_id" name="api_id"
                   placeholder="12345678"
                   value="{{ api_id }}"
                   {% if not is_configured %}inputmode="numeric" required{% endif %}>
        </div>
        <div class="col-12 col-md-6">
            <label for="api_hash" class="form-label">API Hash</label>
            <input type="text" class="form-control" id="api_hash" name="api_hash"
                   placeholder="abcdef1234567890abcdef1234567890"
                   value="{{ api_hash }}"
                   {% if not is_configured %}required{% endif %}>
        </div>
    </div>
    <div class="d-flex justify-content-end">
        <button type="submit" class="btn btn-primary">Сохранить</button>
    </div>
</form>
{% endif %}
