{% for r in records %}
<tr{% if r.ms > 1000 %} class="table-danger"{% elif r.ms > 500 %} class="table-warning"{% endif %}>
  <td><code>{{ r.time|local_dt("time") }}</code></td>
  <td><strong>{{ r.ms }} ms</strong></td>
  <td>{{ r.get("db_ms", "—") }}</td>
  <td>{{ r.get("db_queries", "—") }}</td>
  <td>{{ r.method }}</td>
  <td><code>{{ r.path }}</code></td>
  <td>{{ r.status }}</td>
</tr>
{% else %}
<tr><td colspan="7">Записей нет — выполните несколько запросов к страницам.</td></tr>
{% endfor %}
